#pragma once

#include <QObject>

#include "QGeoCoordinate"

#include "WimaArea.h"

class WimaAreaData
    : public QObject // Abstract class for all WimaAreaData derived objects
{
  Q_OBJECT
public:
  Q_PROPERTY(const QVariantList path READ path NOTIFY pathChanged)
  Q_PROPERTY(QString type READ type CONSTANT)
  Q_PROPERTY(QString mapVisualQML READ mapVisualQML CONSTANT)

  WimaAreaData(QObject *parent = nullptr);
  ~WimaAreaData();
  WimaAreaData(const WimaAreaData &otherData) = delete; // avoid slicing
  WimaAreaData &
  operator=(const WimaAreaData &otherData) = delete; // avoid slicing

  bool operator==(const WimaAreaData &data) const;
  bool operator!=(const WimaAreaData &data) const;

  virtual QString mapVisualQML(void) const = 0;

  QVariantList path() const;
  QGeoCoordinate center() const;
  const QList<QGeoCoordinate> &coordinateList() const;
  bool containsCoordinate(const QGeoCoordinate &coordinate) const;
  virtual QString type() const = 0;

  void append(const QGeoCoordinate &c);
  void push_back(const QGeoCoordinate &c);
  void clear();

signals:
  void pathChanged(const QVariantList &coordinateList);
  void centerChanged(void);

public slots:
  void setPath(const QList<QGeoCoordinate> &coordinateList);
  void setPath(const QVariantList &coordinateList);
  void setCenter(const QGeoCoordinate &center);

protected:
  void assign(const WimaAreaData &other);
  void assign(const WimaArea &other);

private:
  // Member Functions

  // Member Variables
  // see WimaArea.h for explanation
  QVariantList _path;
  mutable QList<QGeoCoordinate> _list;
  QGeoCoordinate _center;
};

bool operator==(const WimaAreaData &m1, const WimaArea &m2);
bool operator!=(const WimaAreaData &m1, const WimaArea &m2);
